/*                                                                            */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <clib/alib_protos.h>
	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/intuition.h>
	#include <reaction/reaction.h>
	#include <reaction/reaction_macros.h>

	#include <proto/window.h>
	#include <classes/window.h>

	#include <proto/layout.h>
	#include <gadgets/layout.h>

	#include <proto/button.h>
	#include <gadgets/button.h>

	#include <string.h>
	#include <stdio.h>

/*----------------------------------------------------------------------------*/
/* Konstanten und Makros                                                      */
/*----------------------------------------------------------------------------*/

	#define GID_END  3


#ifdef __amigaos4__
	#define BASEDEF(base) struct Library *base##Base = NULL; struct base##IFace *I##base = NULL
	#define OPENLIB(base,name) if (!(base##Base = OpenLibrary (name,0)) || !(I##base = (struct base##IFace *)GetInterface ((struct Library *)base##Base,"main",1,NULL))) ok = FALSE
	#define CLOSELIB(base) if (I##base) DropInterface ((struct Interface *)I##base); if (base##Base) CloseLibrary (base##Base)
#else
	#define BASEDEF(base) struct Library *base##Base = NULL
	#define OPENLIB(base,name) if (!(base##Base = OpenLibrary (name,0))) ok = FALSE
	#define CLOSELIB(base) if (base##Base) CloseLibrary (base##Base)
#endif

/*----------------------------------------------------------------------------*/
/* Type definitions                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Globale Variablen                                                          */
/*----------------------------------------------------------------------------*/

	BASEDEF(Window);
	BASEDEF(Layout);
	BASEDEF(Button);

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL open_libs (void)

	{
	BOOL ok = TRUE;

	OPENLIB(Window,"window.class");
	OPENLIB(Layout,"gadgets/layout.gadget");
	OPENLIB(Button,"gadgets/button.gadget");

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void close_libs (void)

	{
	CLOSELIB(Window);
	CLOSELIB(Layout);
	CLOSELIB(Button);
	}

/*----------------------------------------------------------------------------*/
/* Hauptprogramm                                                              */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	Object *winobj;
	struct Screen *scr;
	struct Window *win;
	ULONG winsig,sigs;
	BOOL cont;
	ULONG result;
	UWORD args[4];
	ULONG count = 0;
	struct Gadget *timegad;

	if (!open_libs())
		{
		Printf ("cannot open required BOOPSI classes\n");
		close_libs();
		return (20);
		}

	if (scr = LockPubScreen (NULL))
		{

		if (winobj = WindowObject,
				WA_CustomScreen,scr,
				WA_Title,"ReAct",
				WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_SIZEBBOTTOM | WFLG_SIZEGADGET | WFLG_ACTIVATE,
				WA_IDCMP,IDCMP_VANILLAKEY | IDCMP_INTUITICKS,
				WINDOW_Position,WPOS_CENTERSCREEN,
				WINDOW_ParentGroup,VGroupObject,
					StartHGroup,
						LAYOUT_SpaceOuter,TRUE,
						LAYOUT_AddChild,timegad = ButtonObject,
							GA_Text,"00:00:00.0",
							GA_ReadOnly,TRUE,
							BUTTON_BevelStyle,BVS_NONE,
							BUTTON_Transparent,TRUE,
						EndObject,
					EndHGroup,
					StartHGroup,
						LAYOUT_SpaceOuter,TRUE,
						LAYOUT_FixedVert,FALSE,
						LAYOUT_EvenSize,TRUE,
						LAYOUT_AddChild,
							Button("Ende",GID_END),
					EndHGroup,
				EndVGroup,
			EndWindow)

			{
			if (win = RA_OpenWindow ((Object *)winobj))
				{

				GetAttr (WINDOW_SigMask, winobj, &winsig);

				cont = TRUE;
				do	{
					sigs = Wait (winsig | SIGBREAKF_CTRL_C);

					if (sigs & SIGBREAKF_CTRL_C)
						cont = FALSE;

					if (sigs & winsig)
						while ((result = RA_HandleInput ((Object *)winobj, NULL)) != WMHI_LASTMSG)
							{
							switch (result & WMHI_CLASSMASK)
								{
							case WMHI_INTUITICK:
								count ++;
								args[0] = (count / (10*60*60)) % 100;
								args[1] = (count / (10*60)) % 60;
								args[2] = (count / (10)) % 60;
								args[3] = count % 10;
								SetGadgetAttrs (timegad,win,NULL,GA_Text,"%02d:%02d:%02d:%d",BUTTON_VarArgs,args,TAG_END);
								break;
							case WMHI_GADGETUP:
								switch (result & WMHI_GADGETMASK)
									{
								case GID_END:
									cont = FALSE;
									break;
									}
								break;
							case WMHI_CLOSEWINDOW:
								cont = FALSE;
								break;
							case WMHI_VANILLAKEY:
								if ((result & WMHI_KEYMASK) == 0x1b)
									cont = FALSE;
								break;
							default:
								break;
								}
							}
					}
				while (cont);
				}

			DisposeObject (winobj);
			}

		UnlockPubScreen (NULL,scr);
		}

	close_libs();

	return (0);
	}

/*----------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                       */
/*----------------------------------------------------------------------------*/
